@@ -1,24 +1,25 @@
 #
 # Routines to display a point on a plot during simulations.
 #
-# Author: Andrew Turpin    (aturpin@unimelb.edu.au)
+# Author: Andrew Turpin    (andrew.turpin@lei.org.au)
 # Date: August 2012
 #
-# Copyright 2012 Andrew Turpin
-# This program is part of the OPI (http://perimetry.org/OPI).
-# OPI is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# any later version.
+# Copyright [2022] [Andrew Turpin]
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#   http://www.apache.org/licenses/LICENSE-2.0
 #
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+
+library(grDevices)
 
 # Globals
 # .OpiEnv$SimDisplay$display    # device number of plot, or NA
@@ -59,7 +60,7 @@ simDisplay.setupDisplay <- function(dimensions = NA) {
 # Return
 #   NULL - success
 #     -1 -  setupDisplay has not been called
-#######################################################################################
+#
 simDisplay.setBackground <- function(col, gridCol) { 
     if(!exists("display", envir=.OpiEnv$SimDisplay))
         return(-1)
@@ -88,7 +89,7 @@ simDisplay.setBackground <- function(col, gridCol) {
 ##################################################################################
 # Show stim on plot for duration and wait responseWindow after.
 # No return, just die quietly if neccessary.
-##################################################################################
+#
 simDisplay.present <- function(x, y, color, duration, responseWindow) {
     if(exists("display", envir=.OpiEnv$SimDisplay)) {
         if (!is.na(.OpiEnv$SimDisplay$display)) {
